@@ -6,9 +6,11 @@ import os
 import re
 import socket
 import sre_constants
+import time
 
 from kazoo.client import KazooClient
 from kazoo.exceptions import NoAuthError, NoNodeError
+from kazoo.protocol.states import KazooState
 
 from .util import to_bytes
 
@@ -45,6 +47,12 @@ class AugumentedClient(KazooClient):
         conn = self._connection
         return conn._socket.getpeername() if conn else ""
 
+    @property
+    def client(self):
+        """ the address (ip, port) of the local endpoint """
+        conn = self._connection
+        return conn._socket.getsockname() if conn else ""
+
     @property
     def sessionid(self):
         return "0x%x" % (getattr(self, "_session_id", 0))
@@ -56,6 +64,16 @@ class AugumentedClient(KazooClient):
         """
         return getattr(self, "_protocol_version", 0)
 
+    @property
+    def data_watches(self):
+        """ paths for data watches """
+        return self._data_watchers.keys()
+
+    @property
+    def child_watches(self):
+        """ paths for child watches """
+        return self._child_watchers.keys()
+
     def get(self, *args, **kwargs):
         """ wraps the default get() and deals with encoding """
         value, stat = super(AugumentedClient, self).get(*args, **kwargs)
@@ -290,3 +308,27 @@ class AugumentedClient(KazooClient):
         """ returns `zk://host:port` for the connected host:port """
         host, port = self.address_from_server()
         return "zk://%s:%d" % (host, port)
+
+    def reconnect(self):
+        """ forces a reconnect by shutting down the connected socket
+            return True if the reconnect happened, False otherwise
+        """
+        state_change_event = self.handler.event_object()
+
+        def listener(state):
+            if state is KazooState.SUSPENDED:
+                state_change_event.set()
+
+        self.add_listener(listener)
+
+        self._connection._socket.shutdown(socket.SHUT_RDWR)
+
+        state_change_event.wait(1)
+        if not state_change_event.is_set():
+            return False
+
+        # wait until we are back
+        while not self.connected:
+            time.sleep(0.1)
+
+        return True
