@@ -62,8 +62,9 @@ from .augumented_cmd import (
     Required,
 )
 from .copy import CopyError, Proxy
+from .watcher import get_child_watcher
 from .watch_manager import get_watch_manager
-from .util import Netloc, pretty_bytes, to_bool
+from .util import Netloc, pretty_bytes, to_bool, to_int
 
 
 def connected(func):
@@ -255,23 +256,20 @@ class Shell(AugumentedCmd):
         """
         Recursively watch for all changes under a path.
         examples:
-        watch start /foo/bar [debug]
+        watch start /foo/bar [debug] [childrenLevel]
         watch stop /foo/bar
         watch stats /foo/bar [repeatN] [sleepN]
         """
         wm = get_watch_manager(self._zk)
         if params.command == "start":
-            wm.add(params.path, params.debug.lower() == "true")
+            debug = to_bool(params.debug)
+            children = to_int(params.sleep, -1)
+            wm.add(params.path, debug, children)
         elif params.command == "stop":
             wm.remove(params.path)
         elif params.command == "stats":
-            repeat = 1
-            sleep = 1
-            try:
-                repeat = int(params.debug)
-                sleep = int(params.sleep)
-            except ValueError:
-                pass
+            repeat = to_int(params.debug, 1)
+            sleep = to_int(params.sleep, 1)
             if repeat == 0:
                 while True:
                     wm.stats(params.path)
@@ -283,6 +281,8 @@ class Shell(AugumentedCmd):
         else:
             print("watch <start|stop> <path> [verbose]")
 
+    complete_watch = _complete_path
+
     @ensure_params(Required("src"), Required("dst"),
                    BooleanOptional("recursive"), BooleanOptional("overwrite"),
                    BooleanOptional("async"), BooleanOptional("verbose"),
@@ -627,6 +627,7 @@ class Shell(AugumentedCmd):
         xid=4
         last_zxid=11
         timeout=10000
+        client=('127.0.0.1', 60348)
         server=('127.0.0.1', 2181)
         """
         fmt_str = """state=%s
@@ -635,7 +636,10 @@ protocol_version=%d
 xid=%d
 last_zxid=%d
 timeout=%d
-server=%s"""
+client=%s
+server=%s
+data_watches=%s
+child_watches=%s"""
         self.do_output(fmt_str,
                        self._zk.client_state,
                        self._zk.sessionid,
@@ -643,7 +647,34 @@ server=%s"""
                        self._zk.xid,
                        self._zk.last_zxid,
                        self._zk.session_timeout,
-                       self._zk.server)
+                       self._zk.client,
+                       self._zk.server,
+                       ",".join(self._zk.data_watches),
+                       ",".join(self._zk.child_watches))
+
+    @ensure_params(Optional("match"))
+    def do_history(self, params):
+        """
+        prints the commands history
+
+        example:
+
+        history
+        ls
+        create
+        get /foo
+        get /bar
+
+        history get
+        get /foo
+        get /bar
+        """
+        for hcmd in self.history:
+            if hcmd is None:
+                continue
+
+            if params.match == "" or params.match in hcmd:
+                self.do_output("%s", hcmd)
 
     @ensure_params(Optional("host"))
     def do_mntr(self, params):
@@ -692,6 +723,23 @@ server=%s"""
     def do_sync(self, params):
         self._zk.sync(params.path)
 
+    @connected
+    @ensure_params(Required("path"), BooleanOptional("verbose"))
+    @check_path_exists
+    def do_child_watch(self, params):
+        """
+        watches for child changes for the given path
+
+        example:
+
+        # only prints the current number of children
+        child_watch /
+
+        # prints num of children along with znodes listing
+        child_watch / true
+        """
+        get_child_watcher(self._zk).update(params.path, params.verbose)
+
     @ensure_params(Required("hosts"))
     def do_connect(self, params):
         """
@@ -712,6 +760,14 @@ server=%s"""
         self._disconnect()
         self.update_curdir("/")
 
+    @connected
+    def do_reconnect(self, args):
+        """
+        forces a reconnect by shutting down the connected socket.
+        """
+        self._zk.reconnect()
+        self.update_curdir("/")
+
     @connected
     def do_pwd(self, args):
         self.do_output("%s", self.curdir)
